<?php
declare(strict_types=1);

namespace Inti\Wp\Provider\Plugin;

use Inti\Wp\Plugin\PluginInterface;
use Inti\Wp\Provider\Service\ProviderCollection;
use Inti\Wp\Provider\Type\Term\TaxonomyInterface;
use function add_action;
use function register_taxonomy;

/**
 * Class RegisterTypes
 *
 * @package Inti\Wp\Provider\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
final class RegisterTypes implements PluginInterface
{
    private ProviderCollection $providers;

    public function __construct(ProviderCollection $providers)
    {
        $this->providers = $providers;
    }

    public function run(?array $options = null): void
    {
        add_action('init', [$this, 'init'], 0);
    }

    public function init(): void
    {
        foreach ($this->providers->getProviders() as $provider) {
            $type = $provider->getObjectType();

            if ($type->isBuiltin()) {
                continue;
            }

            if ($type instanceof TaxonomyInterface) {
                $this->register_taxonomy($type);
            }
        }
    }

    private function register_taxonomy(TaxonomyInterface $taxonomy): void
    {
        $parentType = $taxonomy->getParentObjectType();

        register_taxonomy($taxonomy->getKey(), $parentType->getKey(), [
            'description' => $taxonomy->getDescription(),
            'hierarchical' => $taxonomy->isHierarchical(),
            'labels' => $taxonomy->getLabels(),
            'public' => $taxonomy->isPublic(),
            'publicly_queryable' => $taxonomy->isPubliclyQueryable(),
            'show_ui' => $taxonomy->showUi(),
        ]);
    }
}
